//freeware by Joe Nodeland jnodeland@dccnet.com
//feel free to let me know how you use this package.
unit CustomSched;

interface

uses Classes, Controls, Messages, Windows, Forms, Graphics, StdCtrls,
  jwGrid, SysUtils, dialogs, dbtables, uConstants;

const MAXROWS = 99;
const ONEMINUTE = 1 / (60 * 24);
type
  TGroupBy = (gbDate,gbResource);

  //in the dragdrop event, the database is updated with the new date for the
  //event being dragged. The following 2 events happen just before, and just
  //after the database is updated.  This allows the user to update associated
  //records
  TEventDroppedBeforeUpdate = procedure (Sender: TObject; EventId: Longint;
    OriginalStartDate:TDateTime; var OKToMove:Boolean) of object;
  TEventDroppedAfterUpdate = procedure (Sender: TObject; EventId: Longint;
    OriginalStartDate, DroppedStartDate:TDateTime) of object;

  TjwCustomSched = class(TjwGrid)
  private
    FOnDrawCell: TDrawCellEvent;
    fOnEventDroppedBeforeUpdate:TEventDroppedBeforeUpdate;
    fOnEventDroppedAfterUpdate:TEventDroppedAfterUpdate;
    fInformation    :String;
    FReadOnly      :Boolean;
    FUpdating      :Boolean;
    fHourStart     :Double;
    fHourEnd       :Double;
    fHourInterval  :Double;      //number of minutes between each period
    fhrInterval    :Double;      //interval in time units
    fDateRange     :TList;       //list of dates to display
    fColumnDate    :TList;       //list of dates per column
    fResourceRange :TStringList; //list of resources to display
    fRow2Display   :TStringList;
    fGroupBy       :TGroupBy;
    fDateFormat    :String;
    fQuery         :TQuery;
    timeArray      :Variant;
    iTextHeight    :Integer;     //store canvas.textheight
    fBorderFontColor :TColor;
    fFieldID       :String;      //Name of field for unique record id
    fFieldResourceId:String;     //name of field for event resourceid
    fFieldStart    :String;      //name of field for event starttime
    fFieldEnd      :String;      //name of field for event endtime
    fDragId    : Integer;  //id of event being dragged
    procedure SetHourStart(Value: Integer);
    procedure SetHourEnd(Value: Integer);
    procedure SetHourInterval(Value: Integer);
    function GetHourStart:Integer;
    function GetHourEnd:Integer;
    function GetHourInterval:Integer;
    procedure CalcRowNum;
    procedure DisplayFixedCells;
    procedure ChangeFontSize(Inc:Integer; Bold:Boolean);
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
      AState: TGridDrawState); override;
    procedure Paint; override;
    function SelectCell(ACol, ARow: Longint): Boolean; override;
    function CalcEventRect :TRect;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;

    procedure EventDroppedBeforeUpdate(Sender: TObject; EventId: Longint;
    OriginalStartDate:TDateTime; var OKToMove:Boolean); virtual; abstract;
    procedure EventDroppedAfterUpdate(Sender: TObject; EventId: Longint;
      OriginalStartDate, DroppedStartDate:TDateTime); virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    // new to this object ---------------
    procedure ConfigureGrid;
    procedure BuildDisplayLine(var sText:String);Virtual;abstract;
    procedure ExecuteQuery; Virtual;  abstract;
    procedure GetData(var NameId:String; var Start,Stop:TDateTime);
    procedure GetColData(aCol:Integer ;var NameId:String;
                                          var Start,Stop:TDateTime);
    procedure GetColNameId(var theNameId:String; aCol:Integer);
    procedure GetColDate(var aDate:TDateTime; aCol: integer);
    procedure GetColResourceId(var theResourceId:String; aCol:Integer);
    procedure DragOver(Source: TObject; X, Y: Integer;
                           State: TDragState; var Accept: Boolean); override;
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    procedure DragCanceled; override;
    procedure GetSelectedEventID(var ID:Integer);
    // end new --------------------------
    property DragId: Integer read fDragId;
    property DateRange: TList read fDateRange write fDateRange;
    property ResourceRange: TStringList read fResourceRange write fResourceRange;

    property RowCount;
    property ColCount;
    property Col;
    property Row;
    property Selection;
    property RowHeights;
    
  
  published
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;

    property HourStart: Integer read GetHourStart write SetHourStart;
    property HourEnd: Integer read GetHourEnd write SetHourEnd;
    property xInterval: Integer read GetHourInterval write SetHourInterval;
    property Interval: Integer read GetHourInterval write SetHourInterval;

    property FieldId: String read fFieldId write fFieldId;
    property FieldResourceId: String read fFieldResourceId write fFieldResourceId;
    property FieldStart: String read fFieldStart write fFieldStart;
    property FieldEnd: String read fFieldEnd write fFieldEnd;

    property GroupBy: TGroupBy read fGroupBy write fGroupBy;
    property Query: TQuery read fQuery write fQuery;
    property BorderFontColor: TColor read fBorderFontColor write fBorderFontColor
             default clNavy;
    property OnDrawCell: TDrawCellEvent read FOnDrawCell write FOnDrawCell;
    property OnEventDroppedBeforeUpdate: TEventDroppedBeforeUpdate
      read FOnEventDroppedBeforeUpdate write FOnEventDroppedBeforeUpdate;
    property OnEventDroppedAfterUpdate: TEventDroppedAfterUpdate
      read FOnEventDroppedAfterUpdate write FOnEventDroppedAfterUpdate;         
  end;

  //================================================================
  {
  To create your own schedule control, create a descendant of TjwCustomSched
  and implement the following methods
 }

  //================================================================
  TjwDayView = class(TjwCustomSched)
  private
    fResourceQry   :TQuery;
  protected
    
  public
    property Information: String Read fInformation;
    procedure ExecuteQuery; override;
    procedure GetEventId(var EventId:Integer);
    procedure BuildDisplayLine(var sText:String); override;
  published
    property HourStart;
    property HourEnd;
    property Interval;
    property GroupBy;
    property Query;
    property Align;
    property Anchors;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property Canvas;
    property GridLineWidth;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property BorderFontColor;
    property ResourceQuery: TQuery read fResourceQry write fResourceQry;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDock;
    property OnStartDrag;
    property OnResize;
    property OnDrawCell;
    property OnEventDroppedBeforeUpdate;
    property OnEventDroppedAfterUpdate;

  end;

  //procedure Register;
implementation

{procedure Register;
begin
  RegisterComponents('Sample', [TjwDayView]);
end; }

//======================================================================
constructor TjwCustomSched.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  //DefaultDrawing := False;
  Align := alTop;
  RowCount := 6;
  ColCount := 6;
  FixedCols := 1;
  FixedRows := 2;
  fHourInterval := 30;
  timeArray := VarArrayCreate([0, MAXROWS], varVariant);
  ScrollBars := ssNone;
  Options := Options + [goRangeSelect] - [goDrawFocusSelected]
       - [goFixedVertLine] - [goFixedHorzLine]
       - [goVertLine] - [goHorzLine];
  fReadOnly := False;     
  fDateRange := TList.create;
  fColumnDate := TList.Create;
  fResourceRange := TStringList.Create;
  fRow2Display := TStringList.Create;
  fGroupBy := gbDate;
  fBorderFontColor := clNavy;
end;

destructor TjwCustomSched.Destroy;
begin
  fDateRange.Free;
  fResourceRange.Free;
  fRow2Display.Free;
  fColumnDate.Free;
  inherited Destroy;
end;  

procedure TjwCustomSched.WMSize(var Message: TWMSize);
var i,j:Integer;
    aRect: TRect;
begin
  i := Canvas.TextWidth('...c00 am');
  DefaultColWidth := Round(Width  / Colcount);
  DefaultColWidth := Trunc((Width - i) / (Colcount-1)-1);
  DefaultRowHeight := (Height-RowHeights[0]) div (RowCount);
  ColWidths[0] := i;
  ChangeFontSize(+4, True);
  iTextHeight := Canvas.TextHeight('A');
  RowHeights[0] := Round(iTextHeight);
  RowHeights[1] := RowHeights[0];

  If fGroupby = gbDate then
    j := DefaultColWidth * fResourceRange.Count
  else j := DefaultcolWidth  ;
  fDateFormat := 'mmm dd';
  If j < Canvas.textWidth(fDateFormat) then fDateFormat := 'd';
  ChangeFontSize(-4, False);
  iTextHeight := Canvas.TextHeight('A');

  //resize events
  If Query.Active then Query.First;
  With Query do
  While not eof do begin
    aRect := CalcEventRect;
    EventList.EventSetRect(FieldByName(FieldId).asInteger, aRect);
    Next;
  end;
end;

procedure TjwCustomSched.Paint;
begin
  If not fUpdating then begin
    Inherited Paint;
     //display dates and resources across top, and time down side
    DisplayFixedCells;
    //draw the events
    EventList.DrawEvents;
  end;
end;

 //display dates and resources across top, and time down side
procedure tjwCustomSched.DisplayFixedCells;
var i:Integer;
    aRect:TRect;
    TheText,Text2,Text3: string;
    iTime :Double;
    bRect:TRect;
begin
  If fResourceRange.Count = 0 then exit;
  canvas.Brush.Color := clBtnFace;
  canvas.Font.Color := fBorderFontColor;

  //display data in 1st fixed row
  ChangeFontSize(+4, True);
  If fGroupBy = gbDate then begin
    If fDateRange.Count = 1 then begin
      Canvas.TextOut(Colwidths[0], 1,
      FormatDateTime('dd mmm yyyy dddd',Integer(fDateRange[0])));
      Canvas.TextOut(Width - Canvas.textWidth('a')*20, 1,
      FormatDateTime('mmmm',Integer(fDateRange[0])));
    end
    else
    For i := 0 to fDateRange.Count - 1 do begin
      Canvas.TextOut(Colwidths[0]+DefaultColWidth*i*fResourceRange.Count,
      1, FormatDateTime(fDateFormat,Integer(fDateRange[i])));
    end  ;
  end
  else
    For i := 0 to fResourceRange.Count - 1 do begin
      Canvas.Font.Color := gResourceSchedColors[Integer(fResourceRange.Objects[i])];
      Canvas.TextOut(Colwidths[0]+DefaultColWidth*i*fDateRange.Count,
      1, fResourceRange[i]);
      Canvas.Font.Color := fBorderFontColor;
    end;

  // display data in 2nd fixed row
  with ARect, Canvas do
  If (fGroupby = gbResource) and (fDateRange.Count = 1) then
      Canvas.TextOut(Colwidths[0], RowHeights[0],
      FormatDateTime('dd mmm yyyy dddd',Integer(fDateRange[0])))
  else
  For i := 1 to ColCount - 1 do begin
    If fDateRange.Count = 1 then
      Font.Color := gResourceSchedColors[Integer(fResourceRange.Objects[i-1])];
    Textout(ColWidths[0] + DefaultColWidth * (i-1)
                         +(DefaultColWidth-TextWidth(fRow2Display[i]))div 2,
                                                 RowHeights[0],fRow2Display[i]);
    Font.Color := fBorderFontColor;
  end;
  ChangeFontSize(-4, False);
  Font.Color := fBorderFontColor;
  //display time on left side of grid
  With aRect, Canvas do
  For i := FixedRows to RowCount -1 do begin
    aRect := CellRect(0, i);
    bRect:=Rect((Right - Canvas.TextWidth('00..')), Top ,Right ,Bottom);
    itime := timeArray[i-2];
    Text2 := FormatDateTime('nn', iTime);
    If Text2 = '00' then begin
      Text3 := FormatDateTime('h am/pm',iTime);
      TheText := Trim(Copy(Text3,1,Length(Text3)-2));        //hour
      Text3 := Trim(Copy(Text3,Length(Text3)-2,Length(Text3)-1));  //am/pm
    end
    else begin
      TheText := '';
      Text3 := Text2;
    end;

    If TheText <> '' then begin //draw hour
      ChangeFontSize(+4, True);
      TextOut(Right - (Right-Left) div 2 - TextWidth(TheText),
      Top + (Bottom - Top - iTextHeight) div 2 + 3, TheText);
      ChangeFontSize(-4, False);
    end;
    If (Text3='am')or(Text3='pm') then begin
        TextOut( brect.left ,  //draw minute
          Top + (bRect.Bottom - bRect.Top - iTextHeight) div 2+3, Text3);
      Pen.Color := clBtnFace;
      MoveTo(Right, Top);
      LineTo(Right + (ColCount-1) * DefaultColWidth, Top);
    end;
  end;

end;


function TjwCustomSched.SelectCell(ACol, ARow: Longint): Boolean;
begin
 { if ((not FUpdating) and FReadOnly) then
    Result := False
  else }
  Result := inherited SelectCell(ACol, ARow);

end;

procedure TjwCustomSched.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  DrawInfo: TGridDrawInfo;
  CellHit: TGridCoord;
  aRect: TRect;
begin
  if Not Readonly then begin
    CalcDrawInfo(DrawInfo);
    If ssLeft in Shift then begin  //drag operation
      CellHit := CalcCoordFromPoint(X, Y, DrawInfo);
      aRect := CellRect(CellHit.X, CellHit.Y);
      fDragId := fEventList.SetDragging(aRect);
      If fDragId <> -1 then beginDrag(false,10);
    end;
  end;  
  inherited MouseMove(Shift, X, Y);
end;

procedure TjwCustomSched.DrawCell(ACol, ARow: Longint; ARect: TRect;
  AState: TGridDrawState);
var Hold: Integer;
begin
  if Assigned(FOnDrawCell) then begin
    ARect.Left := ClientWidth - ARect.Left;
    ARect.Right := ClientWidth - ARect.Right;
    Hold := ARect.Left;
    ARect.Left := ARect.Right;
    ARect.Right := Hold;
    FOnDrawCell(Self, ACol, ARow, ARect, AState);
  end;
end;

//---------- new methods -------------------------------

{user can drag events around the grid.  Nothing fancy, just drag and
 drop within the source grid.  See the mouseMove event for initiating
 the drag event.}
procedure TjwCustomSched.DragOver(Source: TObject; X, Y: Integer;
                           State: TDragState; var Accept: Boolean);
var CellHit:  TGridCoord;
    DrawInfo: TGridDrawInfo;
begin
  CalcDrawInfo(DrawInfo);
  CellHit := CalcCoordFromPoint(X, Y, DrawInfo);
  if (CellHit.X >= FixedCols) and (CellHit.Y >= FixedRows) then
    Accept := (source is TjwGrid)
  else Accept := False;
end;

procedure TjwCustomSched.DragDrop(Source: TObject; X, Y: Integer);
var CellHit:  TGridCoord;
    DrawInfo: TGridDrawInfo;
    New, Old :TRect;
    NewDate, OldDate, Start, Stop: TDateTime;
    iNumRows:Integer;
    aResourceId, sText: String;
    bOKToMove:Boolean;
begin
  If fDragId >-1 then begin
    CalcDrawInfo(DrawInfo);
    CellHit := CalcCoordFromPoint(X, Y, DrawInfo);
    New := CellRect(CellHit.X, CellHit.Y);
    if EventList.EventGetRect(fDragId, Old) then begin
      New.Left   := Old.Left + (New.Left - Old.Left);
      New.Right  := Old.Right + (New.Right - Old.Right);
      New.Top    := Old.Top + (New.Top - Old.Top);
      New.Bottom := New.Top + (Old.Bottom - Old.top);
      //update database....
      Try
        If Query.Locate(FieldId,fDragId,[]) then begin
          bOKToMove := True;
          OldDate := Query.FieldByName(FieldStart).asDateTime;
          if Assigned(fOnEventDroppedBeforeUpdate) then
            fOnEventDroppedBeforeUpdate(self,fDragId,
            OldDate, bOKToMove);
          if not bOKToMove then exit;
           
          GetColDate(NewDate, CellHit.X);
          GetColResourceId(aResourceId, CellHit.X);
          Start := NewDate + TimeArray[CellHit.Y-FixedRows];
          iNumRows := (New.Bottom - New.Top) div DefaultRowHeight;
          Stop := Start + (iNumRows * (fHrInterval+ONEMINUTE)) - ONEMINUTE;
          Query.Edit;
          Query.FieldByName(FieldResourceId).asString := aResourceId;
          Query.FieldByName(FieldStart).asDateTime := Start;
          Query.FieldByName(FieldEnd).asDateTime := Stop;
          Query.Post;
          if Assigned(fOnEventDroppedAfterUpdate) then
            fOnEventDroppedAfterUpdate(self,fDragId, OldDate,
              Query.FieldByName(FieldStart).asDateTime);

          //update event
          BuildDisplayLine(sText);
          EventList.EventSetText(fDragid,sText);
          EventList.EventSetRect(fDragId, New);
          Windows.InvalidateRect(Handle, @Old, False);
          Windows.InvalidateRect(Handle, @New, False);
        end;
      except on e:exception do
        MessageDlg(e.message,mtError,[mbOK],0);
      end;

    end;
  end;
end;

procedure TjwCustomSched.DragCanceled;
begin
  EventList.CancelDragging(fDragId);
end;

//get the date for the supplied column
procedure TjwCustomSched.GetColDate(var aDate:TDateTime; aCol: integer);
begin
  aDate := Integer(fColumnDate[acol-1]);
end;

// get the resource id fro the supplied column
procedure TjwCustomSched.GetColNameId(var theNameId:String; aCol: integer);
begin
  If fResourceRange.Count = 0 then exit;
  If fGroupBy = gbResource then
    theNameId := fResourceRange[(aCol-1) div fDateRange.Count]
  else theNameId := fRow2Display[aCol]  ;
end;

//get values for 1st selected column
procedure TjwCustomSched.GetData(var NameId:String; var Start,Stop:TDateTime);
begin
  If (fResourceRange.Count = 0) or (fDateRange.Count = 0) then exit;
  GetColData(Selection.Left, NameId, Start, Stop);
end;

procedure TjwCustomSched.GetSelectedEventID(var ID:Integer);
begin
  Id := EventList.FindEvent(CellRect(Col,Row));
end;

//get values for supplied Column.  Used when a range of cells is selected
procedure TjwCustomSched.GetColData(aCol:Integer ;var NameId:String;
                                          var Start,Stop:TDateTime);
var DatePortion: TDateTime;
begin
  If (fResourceRange.Count = 0) or (fDateRange.Count = 0) then exit;
  GetColNameId(NameId, aCol);
  GetColDate(DatePortion, aCol);
  Start := DatePortion + TimeArray[Selection.Top-FixedRows];
  Stop := DatePortion + TimeArray[Selection.Bottom-FixedRows] + fHrInterval;
end;

// get the resourceid for the supplied column
procedure TjwCustomSched.GetColResourceId(var theResourceId:String; aCol: integer);
begin
  If fResourceRange.Count = 0 then exit;
  If GroupBy = gbResource then
    theResourceId := fResourceRange[(aCol-1) div fDateRange.Count]
  else theResourceId := fRow2Display[aCol]  ;
end;

procedure tjwCustomSched.ChangeFontSize(Inc:Integer; Bold:Boolean);
begin
  canvas.Font.Size := Font.Size + Inc;
  If Bold then
    canvas.font.Style := Font.Style + [fsBold]
  else
    canvas.font.Style := Font.Style - [fsBold] ;
end;

function TjwCustomSched.CalcEventRect :TRect;
var aCol, i, iDateIndex, iStartRow, iResourceIndex: LongInt;
    iNumRows: Integer;
    dTemp: Double;
begin
  With query do begin
    iDateIndex := -1;
    //find the column to draw in
    iResourceIndex := fResourceRange.IndexOf(FieldByName(FieldResourceId).asString);
    If iResourceIndex <> -1 then iResourceIndex := iResourceIndex + 1;
    For i := 0 to fDateRange.Count-1 do
      If Trunc(FieldByName(FieldStart).asDateTime)
             = Integer(fDateRange[i]) then begin
        iDateIndex := i;
        Break;
      end;

    //find the starting and ending row
    dTemp := FieldByName(FieldStart).asFloat;
    iStartRow := Round((dTemp - Trunc(dTemp)- fHourStart)/ (fHrInterval+ONEMINUTE))
                 + FixedRows;
    //If iStartRow < FixedRows then iStartRow := FixedRows + 1;             
    iNumRows := Round((FieldByName(FieldEnd).asFloat
                 - FieldByName(FieldStart).asFloat)
                 / (fHrInterval+ONEMINUTE));

    //build the result
    If (iResourceIndex<>-1) and (iDateIndex<>-1)and(iStartRow<>-1) then begin
      If fGroupBy = gbDate then
        aCol := iResourceIndex + (iDateIndex * fResourceRange.Count)
      else
        aCol := ((iResourceIndex-1) * fDateRange.Count)+(iDateIndex+1);
      Result := CellRect(aCol, iStartRow);
      Result.Bottom := Result.Top + (iNumRows * DefaultRowHeight);
    end;
  end;  
end;

//calculate what to display in the top 2 rows of the grid, and refresh query
procedure TjwCustomSched.ConfigureGrid;
var i,j:integer;
    iTime:TTime;
begin
  If fResourceRange.Count = 0 then exit;
  Try
    fUpdating := True;
    ColCount := fResourceRange.Count * fDateRange.Count + 1;
    //force grid sizing
    Perform(wm_size,0,0);
  Finally
    fUpdating := False;
  End;

  //calc data for fixed columns  (save time to use in query.filter)
  for i := 0 to Rowcount - FixedRows  do begin
    iTime := (fHourStart) + ((i)/fHourInterval);
    timeArray[i] := iTime;
  end;

  //calc data for fixed rows
  fColumnDate.Clear;
  fRow2Display.Clear;
  fRow2Display.Add('');
  If fGroupBy = gbDate then
    For i := 0 to fDateRange.Count - 1 do begin
      For j := 0 to fResourceRange.Count - 1 do begin
        fColumnDate.Add(Pointer(fDateRange[i]));
        fRow2Display.Add(fResourceRange[j]);
      end;
    end
  else For i := 0 to fResourceRange.Count - 1 do begin
      For j := 0 to fDateRange.Count - 1 do begin
        fColumnDate.Add(Pointer(fDateRange[j]));
        fRow2Display.Add(FormatDateTime(fDateFormat,Integer(fDateRange[j])));
      end;
    end;

  //Fill eventlist with events
  ExecuteQuery;

end;

procedure TjwCustomSched.CalcRowNum;
var iRowCount, iInterval:Integer;
begin
  iInterval := GetHourInterval;
  If (iInterval = 0) or (fHourEnd = 0) or (fHourStart = 0) then exit;
  iRowCount := Trunc((HourEnd - HourStart) * (60 Div iInterval))+FixedRows;
  If RowCount <> iRowCount then  RowCount := iRowCount;
end;

procedure TjwCustomSched.SetHourStart(Value: Integer);
begin
  fHourStart := Value/24;
  CalcRowNum;
end;

function TjwCustomSched.GetHourStart:Integer;
begin
  Result := Trunc(fHourStart * 24);
end;

procedure TjwCustomSched.SetHourEnd(Value: Integer);
begin
  fHourEnd := (Value)/24;
  CalcRowNum;
end;

function TjwCustomSched.GetHourEnd:Integer;
begin
  Result := Trunc(fHourEnd * 24);
end;

procedure TjwCustomSched.SetHourInterval(Value: Integer);
begin
  fHourInterval := (60 / Value) * 24;
  fhrInterval := (Value-1) / 1440;
  CalcRowNum;
end;

function TjwCustomSched.GetHourInterval:Integer;
begin
  Result := Trunc(24/fHourInterval*60);
end;




////////////////////////////// ///////////////////////////////////
{
  To create your own schedule control, create a descendant of TjwCustomSched
  and implement the following methods
 }
///////////////// TjwDayView ///////////////////////////////////
//builds line for displaying inside event;
procedure TjwDayView.BuildDisplayLine(var sText:String);
begin
  sText := '';
  With Query do begin
    sText := FieldbyName('NameId').asString + Chr(255);
    sText := sText + FieldbyName('PhoneNumber').asString + Chr(255);
    sText := sText + FormatDateTime('hh:mm am/pm',FieldbyName('StartDate').asDateTime);
  end;
end;

//build 'where' string from date range and resource range, then execute query
procedure TjwDayView.ExecuteQuery;
var i:Integer;
    sText:String;
    aRect, bRect: TRect;
begin
  If (fDateRange.Count = 0) or (fResourceRange.Count = 0) then exit;
  With Query do begin
    Close;
    If sql.count = 1 then sql.Add('');
    If sql.count = 2 then sql.Add('');
    sql[1] := 'where (';
    sql[2] := '';

    For i := 0 to fDateRange.Count -1 do begin
      sql[2] := sql[2] + ' StartDate >= :StartDate'+ intTostr(i)
      + ' and EndDate < :EndDate' + IntToStr(i) + ' or ';
      Parambyname('StartDate'+IntToStr(i)).AsDateTime := Integer(fDateRange[i]);
      Parambyname('EndDate'+IntToStr(i)).AsDateTime :=  Integer(fDateRange[i])+1;
    end;
    sql[2] := Copy(sql[2],1, length(sql[2]) - length(' or ')) + ') and (';
    For i := 0 to fResourceRange.Count -1 do begin
      sql[2] := sql[2] + ' ResourceId = :Id'+ IntToStr(i) + ' or ';
      Parambyname('Id'+IntToStr(i)).AsString := fResourceRange[i];
    end;
    sql[2] := Copy(sql[2],1, length(sql[2]) - length(' or ')) + ')';
    Open;

    //populate the event list
    Try
      EventList.ClearAll;
      While not eof do begin
        aRect := CalcEventRect;
        bRect := CellRect(0,FixedRows);
        If aRect.Top >= bRect.Top then begin
          BuildDisplayLine(sText);
          EventList.EventAdd(FieldByName('Id').asInteger,
                            sText,
                            FieldByName('EventState').asInteger,
                            FieldByName('FontColor').asInteger,
                            FieldByName('BackColor').asInteger,
                            aRect);
        end;
        Next;
      end;
    Except on e:exception do
      MessageDlg(e.message,mtError,[mbOK],0);
    End;
  end;

  //If there is 1 date selected get the scheduled status of the resource
  for i := 0 to fResourceRange.Count-1 do
    fResourceRange.Objects[i] := nil;
  If fDateRange.Count = 1 then
    With ResourceQuery do begin
      Params[0].asDateTime := Integer(fDateRange[0]);
      Open;
      While not eof do begin
        for i := 0 to fResourceRange.Count-1 do
          If fResourceRange[i] = FieldByName('ResourceId').asString then
            fResourceRange.Objects[i] := Pointer(FieldByName('SchedStatus').asInteger);
        Next;
      end;
      Close;
    end;
end;

procedure TjwDayView.GetEventId(var EventId:Integer);
begin
  GetSelectedEventID(EventId);
  If EventId = -1 then exit;
  With Query do begin
    First;
    While not eof do begin
      If (EventId = FieldByName('Id').asInteger) then begin
        fInformation := 'ResourceId: ' +FieldByname('Id').asString
                       +'  Event: ' +FieldByName('NameId').asString ;
        Exit;
      end;
      Next;
    end;
  end;
end;


end.
