unit RowCal;
//freeware by Joe Nodeland jnodeland@dccnet.com
//feel free to let me know how you use this package.

{This unit is hacked from the Borland sample calendar.  Could be improved
greatly, but time contraints say otherwise...}

interface

uses Classes, Controls, Messages, Windows, Forms, Graphics, StdCtrls,
  Grids, SysUtils, dialogs;

type
  TDayOfWeek = 0..6;

  TRowCal = class(TCustomGrid)
  private
    FDate: TDateTime;
    fSelectedDate: TDateTime;
    fNewDate:TDate;
    fSelectedList:TStringList;
    fFontColor:TColor;
    FMonthOffset: Integer;
    FStartOfWeek: TDayOfWeek;
    fStartDate:TDateTime;  //first displayed date on calendar
    FOnChange: TNotifyEvent;
    FReadOnly: Boolean;
    FUpdating: Boolean;
    FUseCurrentDate: Boolean;
    fDisplayRows: Integer;
    fDisplayCols: Integer;

    function GetCellText(ACol, ARow: Integer): string;
    function GetDateElement(Index: Integer): Integer;
    procedure SetRowCalDate(Value: TDateTime);
    procedure SetDateElement(Index: Integer; Value: Integer);
    procedure SetStartOfWeek(Value: TDayOfWeek);
    procedure SetUseCurrentDate(Value: Boolean);
    function StoreCalendarDate: Boolean;
    procedure SetDisplayRows(Value: Integer);
    procedure SetDisplayCols(Value: Integer);
    function GetDisplayRows: Integer;
    function GetDisplayCols: Integer;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    procedure Change; dynamic;
    procedure ChangeMonth(Delta: Integer);
    procedure Click; override;
    function DaysPerMonth(AYear, AMonth: Integer): Integer; virtual;
    function DaysThisMonth: Integer; virtual;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
                       AState: TGridDrawState); override;
    function IsLeapYear(AYear: Integer): Boolean; virtual;
    function SelectCell(ACol, ARow: Longint): Boolean; override;
    procedure UpdateSelectedList(aDate:TDate);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property SelectedDate: TDateTime  read fSelectedDate;
    property CalendarDate: TDateTime  read FDate write SetRowCalDate
                                           stored StoreCalendarDate;
    property CellText[ACol, ARow: Integer]: string read GetCellText;
    property DisplayRows: Integer read fDisplayRows write SetDisplayRows;
    property DisplayCols: Integer read fDisplayCols write SetDisplayCols;
    property SelectedDates: TStringList read fSelectedList;  
    procedure NextMonth;
    procedure NextYear;
    procedure PrevMonth;
    procedure PrevYear;
    procedure CalcMonthOffset; virtual;
    property RowCount;
    property ColCount;
    property Col;
    property Row;
    property Selection;

  published
    //property RowsDiplayed: Integer read fRowCount write setRowsToDisplay;
    //property ColumnsDisplayed: Integer read fColCount write setDaysAcross;
    property Align;
    property Anchors;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property Day: Integer index 3  read GetDateElement write SetDateElement
                                   stored False;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property GridLineWidth;
    property Month: Integer index 2  read GetDateElement write SetDateElement
              stored False;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property StartOfWeek: TDayOfWeek read FStartOfWeek write SetStartOfWeek;
    
    property ShowHint;
    property TabOrder;
    property TabStop;
    property UseCurrentDate: Boolean read FUseCurrentDate write SetUseCurrentDate
                             default True;
    property Visible;
    property Year: Integer index 1  read GetDateElement write SetDateElement
             stored False;
    property OnClick;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDock;
    property OnStartDrag;
    property OnResize;
  end;
  procedure Register;
implementation

procedure Register;
begin
  RegisterComponents('Sample', [TRowCal]);
end;

constructor TRowCal.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { defaults }
  fSelectedList:=TStringList.Create;
  fSelectedList.Sorted := True;
  FUseCurrentDate := True;
  Align := alTop;
  FixedCols := 0;
  FixedRows := 1;
  fDisplayRows := 2;
  fDisplayCols := 35;
  RowCount := fDisplayRows;
  ColCount := fDisplayCols;
  ScrollBars := ssNone;
  Options := Options{ - [goRangeSelect]} + [goDrawFocusSelected];
  CalendarDate := Date;
  fSelectedDate := fDate;

  CalcMonthOffset;

end;

destructor TRowCal.Destroy;
begin
  fSelectedList.Free;
  inherited Destroy;
end;  

procedure TRowCal.Change;
var i,j:Integer;
begin
  fSelectedList.Clear;
  For i := Selection.Left to Selection.Right do
   For j := Selection.Top to Selection.Bottom do begin
     fNewDate := fStartDate + i + (j - 1)* fDisplayCols;
    UpdateSelectedList(fNewDate);
   end;
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TRowCal.Click;
  var aYear,aMonth,aDay:Word ;
begin
  inherited Click;
 DecodeDate(fStartDate + Col + (Row - 1)
             * fDisplayCols, aYear, aMonth, aDay);
  fSelectedDate :=EncodeDate(aYear,aMonth,aDay);
  change;
end;

procedure TRowCal.SetDisplayRows(Value: Integer);
begin
  If RowCount <> Value then begin
    fDisplayRows := Value;
    RowCount := Value;
    Resize;
    Invalidate;
  end;
end;

procedure TRowCal.SetDisplayCols(Value: Integer);
begin
  If ColCount <> Value then begin
    fDisplayCols := Value;
    ColCount := Value;
    //Resize;
    //invalidate;
  end;
end;

function TRowCal.GetDisplayRows: Integer;
begin
  GetDisplayRows := fDisplayRows;
end;

function TRowCal.GetDisplayCols: Integer;
begin
  GetDisplayCols := fDisplayCols;
end;  

function TRowCal.IsLeapYear(AYear: Integer): Boolean;
begin
  Result := (AYear mod 4 = 0) and ((AYear mod 100 <> 0) or (AYear mod 400 = 0));
end;

function TRowCal.DaysPerMonth(AYear, AMonth: Integer): Integer;
const
  DaysInMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31,
  30, 31);
begin
  Result := DaysInMonth[AMonth];
  if (AMonth = 2) and IsLeapYear(AYear) then Inc(Result); { leap-year Feb is special }
end;

function TRowCal.DaysThisMonth: Integer;
begin
  Result := DaysPerMonth(Year, Month);
end;

procedure TRowCal.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var TheText: string;
begin
  TheText := CellText[ACol, ARow];
  If (aCol >= Selection.Left)and(aCol <= Selection.Right)
  and (aRow>= Selection.Top) and (aRow <= Selection.Bottom) then begin
    Canvas.Brush.Color := clBlue;
    Canvas.Font.Color := clWhite;
    fNewDate := fStartDate + aCol + (aRow - 1) * fDisplayCols;
  end
  else if aRow > 0 then begin
    Canvas.Brush.Color := clWindow;
  end
  else if aRow = 0 then begin
    Canvas.Brush.Color := clBtnFace;
    Canvas.Font.Color := fFontColor;
  end;

  with ARect, Canvas do
    TextRect(ARect, Left + (Right - Left - TextWidth(TheText)) div 2,
      Top + (Bottom - Top - TextHeight(TheText)) div 2, TheText);
end;

function TRowCal.GetCellText(ACol, ARow: Integer): string;
var
  DayNum: Integer;
  AYear, AMonth, ADay, DecDays, iDaysPerMonth:Word;
  aDate: TDateTime;
begin
  aDate := Now;
  if ARow = 0 then begin { day names at tops of columns }
    
    //change request - always show calendar starting on 1st day of month
    //so comment out next line, and add 2nd line
    //Result := ShortDayNames[(StartOfWeek + ACol) mod 7 + 1];
    Result := FormatDateTime('ddd',fStartDate + aCol);

    If Width div ColCount < Canvas.TextWidth('Wed') then
      Result := Copy(Result,1,1);
  end  
  else
  begin
    DayNum := FMonthOffset + ACol + (ARow - 1)  * fDisplayCols;
    If (aRow=1)and (DayNum < 1) then begin
      iDaysPerMonth := DaysPerMonth(Year,Month);
      aDate := IncMonth(EncodeDate(Year,Month,iDaysPerMonth),-1);
      DecodeDate(aDate, AYear, AMonth, ADay);
      DayNum := DaysPerMonth(aYear,aMonth) + DayNum;
      Result := IntToStr(DayNum);
      Canvas.Font.Color := clGray;
    end
    else
    if (DayNum > DaysThisMonth) then begin
      DecDays := 0;
      aYear := Year;
      aMonth := Month;
      Repeat
        iDaysPerMonth := DaysPerMonth(aYear,aMonth);
        DecDays := DecDays + iDaysPerMonth;
        Try
        aDate := IncMonth(EncodeDate(aYear,aMonth, iDaysPerMonth), 1);
        except
          showmessage(inttoStr(ayear) + ' ' + inttostr(amonth) +
          ' ' + intToStr(idayspermonth));
        end;  
        DecodeDate(aDate, AYear, AMonth, ADay);
        iDaysPerMonth := DaysPerMonth(aYear,aMonth);
      Until DayNum-DecDays <= iDaysPerMonth;
      Result := IntToStr(DayNum-DecDays);
      Canvas.Font.Color := clGray;
    end
    else begin
      Result := IntToStr(DayNum);
      Canvas.Font.Color := clNavy;
    end;
  end;
end;
                      
procedure TRowCal.UpdateSelectedList(aDate:TDate);
begin
  If (Selection.Left = Selection.Right)
  and(Selection.Top = Selection.Bottom) then
   FSelectedList.Clear;
  FSelectedList.Add(IntToStr(Trunc(aDate)));
end;

function TRowCal.SelectCell(ACol, ARow: Longint): Boolean;
begin
  if ((not FUpdating) and FReadOnly) or (CellText[ACol, ARow] = '') then
    Result := False
  else Result := inherited SelectCell(ACol, ARow);

end;

procedure TRowCal.SetRowCalDate(Value: TDateTime);
var  AYear, AMonth, ADay: Word;
begin
  DecodeDate(Value, AYear, AMonth, ADay);
  If (aMonth <> Month) or (aYear <> Year) then begin
    FDate := Value;
    CalcMonthOffset;
    Refresh;
  end;
  FDate := Value;
  Row := (Day - fMonthOffset) div ColCount + 1;
  Col := (Day - fMonthOffset) mod ColCount;
  Change;
end;

function TRowCal.StoreCalendarDate: Boolean;
begin
  Result := not FUseCurrentDate;
end;

function TRowCal.GetDateElement(Index: Integer): Integer;
var
  AYear, AMonth, ADay: Word;
begin
  DecodeDate(FDate, AYear, AMonth, ADay);
  case Index of
    1: Result := AYear;
    2: Result := AMonth;
    3: Result := ADay;
    else Result := -1;
  end;
end;

procedure TRowCal.SetDateElement(Index: Integer; Value: Integer);
var
  AYear, AMonth, ADay: Word;
begin
  if Value > 0 then
  begin
    DecodeDate(FDate, AYear, AMonth, ADay);
    case Index of
      1: if AYear <> Value then AYear := Value else Exit;
      2: if (Value <= 12) and (Value <> AMonth) then AMonth := Value else Exit;
      3: if (Value <= DaysThisMonth) and (Value <> ADay) then ADay := Value else Exit;
      else Exit;
    end;
    FDate := EncodeDate(AYear, AMonth, ADay);
    FUseCurrentDate := False;
    CalcMonthOffset;
   // Change;
  end;
end;

procedure TRowCal.SetStartOfWeek(Value: TDayOfWeek);
begin
 
end;

procedure TRowCal.SetUseCurrentDate(Value: Boolean);
begin
  if Value <> FUseCurrentDate then
  begin
    FUseCurrentDate := Value;
    if Value then
    begin
      FDate := Date; { use the current date, then }
      CalcMonthOffset;
    end;
  end;
end;

procedure TRowCal.CalcMonthOffset;
var
  AYear, AMonth, ADay, iDaysPerMonth: Word;
  FirstDate: TDateTime;

begin
  FUpdating := True;
  try
    DecodeDate(FDate, aYear, aMonth, aDay);
    FirstDate := EncodeDate(AYear, AMonth, 1);

    //change request - force the start date to always = 1st of the month
    //so comment out the stuff below
    {FMonthOffset := 2 - ((DayOfWeek(FirstDate) - StartOfWeek + 7) mod 7); // day of week for 1st of month
    if FMonthOffset = 2 then FMonthOffset := -5;
    If fMonthOffset = 1 then
      fStartDate := FirstDate
    else begin
      iDaysPerMonth := DaysPerMonth(aYear,aMonth);
      FirstDate := IncMonth(EncodeDate(aYear,aMonth,iDaysPerMonth),-1);
      DecodeDate(FirstDate, AYear, AMonth, ADay);
      iDaysPerMonth := DaysPerMonth(aYear,aMonth) + FMonthOffset;
      fStartDate := EncodeDate(aYear,aMonth,iDaysPerMonth);
    end; }
  finally

    //change request - force the start date to always = 1st of the month
    fStartDate := FirstDate;
    fMonthOffset := 1;
    {To start the calendar on a date other than the 1st do the following
    fStartDate := FirstDate-15;
    fMonthOffset := -14;}

    FUpdating := False;
  end;
end;

procedure TRowCal.ChangeMonth(Delta: Integer);
begin
  fDate := incMonth(fDate,Delta);
  CalcMonthOffset;
  Invalidate;
  Click;
end;

procedure TRowCal.PrevMonth;
begin
  ChangeMonth(-1);
end;

procedure TRowCal.NextMonth;
begin
  ChangeMonth(1);
end;

procedure TRowCal.NextYear;
begin
  if IsLeapYear(Year) and (Month = 2) and (Day = 29) then Day := 28;
  Year := Year + 1;
end;

procedure TRowCal.PrevYear;
begin
  if IsLeapYear(Year) and (Month = 2) and (Day = 29) then Day := 28;
  Year := Year - 1;
end;


procedure TRowCal.WMSize(var Message: TWMSize);
begin
  DefaultColWidth := Trunc(Message.Width / (Colcount+2));
  DefaultRowHeight := (Message.Height) div fDisplayRows;
end;

end.

