//freeware by Joe Nodeland jnodeland@dccnet.com or jedaworks@dccnet.com
//feel free to let me know how you use this package.

{UEventList unit defines the object that stores an events data (TjwEvent)
and the ojbect that holds all the events in a list (TjwEventList).  When a
scheduled event is created, a TjwEvent object is created and added to the
TjwEventList.

TjwEventList is a property of the grid object TjwGrid.

TjwGrid is a remake of the Borland TCustomGrid, with a few items changed,
mostly in the drawing routines.

TjwCustomSched descends from TjwGrid, and impliments most of the additional
behavior required for the schedule. To extend the component, modify the code
in this unit.  But remember, any code modified here will affect all schedules.

Users descend their own grid from TjwCustomSched and impliment the following
methods.
TjwDayView.BuildDisplayLine
TjwDayView.ExecuteQuery
TjwDayView.GetEventId

See TjwDayView in unit CustomSched for code that is used in the sample.
}
unit dMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBTables, Db,  uConstants,dbctrls,buttons,stdctrls, menus
  ,comctrls, extctrls, Grids;

function tapiRequestMakeCall(lpszDestAddress, lpszAppName, lpszCalledParty, lpszComment: LPCSTR): DWORD; stdcall;

type TConfiguration = record
  StartHour :Integer;
  EndHour   :Integer;
  Interval  :Integer;
end;

type
  TdmMain = class(TDataModule)
    Database1: TDatabase;
    qryTemp: TQuery;
    qryResource: TQuery;
    procedure dmMainCreate(Sender: TObject);
    procedure qryResourceBeforePost(DataSet: TDataSet);
    procedure qryResourceAfterDelete(DataSet: TDataSet);
    procedure qryResourceBeforeDelete(DataSet: TDataSet);
    procedure qryResourceAfterInsert(DataSet: TDataSet);
    procedure qryResourceBeforeEdit(DataSet: TDataSet);
  private

  public
    OriginalResourceId:String;//id before edit;
    ResourceMode:String;  // is a resource being inserted or edited?, see fResources
    Configuration:TConfiguration;
    procedure GetConfiguration;
    procedure SetConfiguration;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
                        Shift: TShiftState);

    // phone dialer
    procedure DialPhone(aPhoneNumber, aCalledParty:String);
  end;

var
  dmMain: TdmMain;

implementation

 uses fMain;
{$R *.DFM}
function tapiRequestMakeCall; external 'TAPI32.DLL' name 'tapiRequestMakeCall';

procedure TdmMain.dmMainCreate(Sender: TObject);
begin
  with Database1 do
  Try
    Connected := True;
    GetConfiguration;

    //populate table with some data....
    With qryTemp do begin
      sql.Text := 'Select Count(*) from resourceSched where startdate = :date';
      Params[0].asDateTime := Date;
      Open;
      If Fields[0].asInteger = 0 then begin
        Close;
        sql.Text := 'Insert into resourceSched (StartDate,ResourceId,SchedStatus) '
                 +  ' Values(:Start,:Id,:Status)';
        Params[0].asDateTime := now;
        Params[1].asString := 'R1';
        Params[2].asInteger := 1;
        execSql;
        Params[0].asDateTime := now;
        Params[1].asString := 'R2';
        Params[2].asInteger := 2;
        execSql;
      end
      else close;
    end;
    
    Application.CreateForm(TfrmMain, frmMain);

  except on e:exception do begin
      Connected := False;
      MessageDlg(e.message,mtError,[mbOK],0);
    end;
  end;

end;  

//------------------- Resource methods -----------------------------

procedure TdmMain.qryResourceBeforePost(DataSet: TDataSet);
begin
  With Dataset do begin
    If Fieldbyname('ResourceId').asString = '' then begin
      raise Exception.Create('A Resource Id is required');
    end;
  end;
  //If updating existing record then cascade changes
  If (ResourceMode='Edit')
  and(OriginalResourceId <> Dataset.FieldByName('ResourceId').asString)
  and(Dataset.FieldByName('IsTruck').asInteger = 1) then
  With qryTemp do
  try
    Database1.StartTransaction;
    sql.Text := 'Update Trucks set resourceid = :id where resourceid = :id2';
    params[0].asString := Dataset.FieldbyName('ResourceId').asString;
    params[1].asString := OriginalResourceId;
    Execsql;
    sql.Text := 'Update Events set resourceid = :id where resourceid = :id2';
    params[0].asString := Dataset.FieldbyName('ResourceId').asString;
    params[1].asString := OriginalResourceId;
    Execsql;
    sql.Text := 'Update ResourceSched set resourceid = :id where resourceid = :id2';
    params[0].asString := Dataset.FieldbyName('ResourceId').asString;
    params[1].asString := OriginalResourceId;
    Execsql;
    Database1.Commit;
  except on e:exception do begin
      If Database1.InTransaction then Database1.Rollback;
     // MessageDlg(e.message,mtError,[mbOK],0);
      Raise;
    end;
  end;

  //if inserting, check if ResourceId already used
  If (ResourceMode='Insert')then
  With qryTemp do begin
    sql.Text := 'Select ResourceId from Resources  where resourceid = :id';
    params[0].asString := Dataset.FieldbyName('ResourceId').asString;
    Open;
    If not Fields[0].isNull then
      Raise Exception.Create('The entered Resource Id has already been used.');
  end;

  //If inserting truck, create truck record if needed
  If (ResourceMode='Insert')
  and(Dataset.FieldByName('IsTruck').asInteger = 1) then
  With qryTemp do
  try
    sql.Text := 'Select ResourceId from Trucks  where resourceid = :id';
    params[0].asString := Dataset.FieldbyName('ResourceId').asString;
    Open;
    If Fields[0].isNull then begin
      sql.Text := 'Insert into Trucks (resourceid) values(:id)';
      params[0].asString := Dataset.FieldbyName('ResourceId').asString;
      Execsql;
    end;
  except on e:exception do begin
      //MessageDlg(e.message,mtError,[mbOK],0);
      Raise;
    end;
  end;
end;

procedure TdmMain.qryResourceBeforeEdit(DataSet: TDataSet);
begin
  OriginalResourceId := Dataset.FieldByName('ResourceId').asString;
end;

procedure TdmMain.qryResourceBeforeDelete(DataSet: TDataSet);
begin
  If (Dataset.FieldByName('IsTruck').asInteger = 1)
  and (MessageDlg('Delete associated Truck records from system?',
                 mtConfirmation,[mbYes,mbNo],0)
  = mrYes) then  With qryTemp do
    try
      sql.Text := 'Delete from Trucks  where resourceid = :id';
      params[0].asString := Dataset.FieldbyName('ResourceId').asString;
      ExecSql;
    except on e:exception do begin
        MessageDlg(e.message,mtError,[mbOK],0);
        Raise;
      end;
    end;
end;

procedure TdmMain.qryResourceAfterInsert(DataSet: TDataSet);
begin
   Dataset.FieldByName('IsTruck').asInteger := 1;
end;

procedure TdmMain.qryResourceAfterDelete(DataSet: TDataSet);
begin

end;

//------------------------------------------------------------

//get configuratio parameters from database
procedure TdmMain.GetConfiguration;
begin
  With qryTemp, Configuration do
  Try
    sql.Text := 'Select * from Configuration';
    Open;
    StartHour := FieldbyName('StartHour').asInteger;
    If StartHour = 0 then StartHour := 8;
    EndHour := FieldbyName('EndHour').asInteger;
    If EndHour = 0 then EndHour := 16;
    Interval := FieldbyName('HourInterval').asInteger;
    If Interval = 0 then Interval := 30;
  Finally
    Close;
  End;
end;

//set configuratio parameters into database
procedure TdmMain.SetConfiguration;
begin
  With qryTemp do
  Try
    sql.Text := 'Update Configuration set StartHour = :Start'
              + ',EndHour = :end '
              + ',HourInterval = :Interval';
    Params[0].asInteger := Configuration.StartHour;
    Params[1].asInteger := Configuration.EndHour;
    Params[2].asInteger := Configuration.Interval;
    ExecSql;
  Finally
   
  End;
end;

procedure TdmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    Ord('X'),ord('x'): If (ssAlt in Shift)
    and (Sender is TForm) then TForm(Sender).Close;
  end;
end;


procedure InitStartInfo(var StartInfo: TStartupInfo);
begin
  StartInfo.cb := sizeof(TStartupInfo);
  StartInfo.lpReserved:= NIL;
  StartInfo.lpDesktop := NIL;
  StartInfo.lpTitle   := NIL;
  StartInfo.dwX    :=0;
  StartInfo.dwY    :=0;
  StartInfo.dwXSize:=400;
  StartInfo.dwYSize:=600;
  StartInfo.dwXCountChars  :=0;
  StartInfo.dwYCountChars  :=0;
  StartInfo.dwFillAttribute:=0;
  StartInfo.dwFlags    :=STARTF_USESHOWWINDOW;
  StartInfo.wShowWindow:=SW_SHOWNORMAL;
  StartInfo.cbReserved2:=0;
  StartInfo.lpReserved2:=NIL;
  StartInfo.hStdInput  :=0;
  StartInfo.hStdOutput :=0;
  StartInfo.hStdError  :=0;
end;

procedure TdmMain.DialPhone(aPhoneNumber, aCalledParty:String);
var
  lResult: longint;
  PhoneNumber: array [0..255] of char;
  Called: array [0..255] of char;
  StartInfo: TStartupInfo;
  ProcInfo:TProcessInformation;
  Windir:String;
  Buffer: array[0..1023] of Char;
begin
  If aPhoneNumber <> '' then begin
    StrPCopy(PhoneNumber,aPhoneNumber);
    StrPCopy(Called,aCalledParty);
    lResult:=tapiRequestMakeCall(PhoneNumber,'',Called,'');
  end  
  else begin
    InitStartInfo(StartInfo);
    GetWindowsDirectory(Buffer,sizeof(Buffer));
    Windir := Buffer;
    If Windir[length(Windir)] <> '\' then Windir := Windir + '\';
    CreateProcess(PChar(Windir + 'Dialer.exe'), NIL, NIL, NIL, False,
                      CREATE_DEFAULT_ERROR_MODE, NIL, NIL, StartInfo,
                      ProcInfo);
  end;                        
end;



end.
