unit uEventList;
interface

//freeware by Joe Nodeland jnodeland@dccnet.com
//feel free to let me know how you use this package.

{UEventList unit defines the object that stores an events data (TjwEvent)
and the ojbect that holds all the events in a list (TjwEventList).  When a
scheduled event is created, a TjwEvent object is created and added to the
TjwEventList.

TjwEventList is a property of the grid object TjwGrid.

TjwGrid is a remake of the Borland TCustomGrid, with a few items changed,
mostly in the drawing routines.

TjwCustomSched descends from TjwGrid, and implements most of the additional
behavior required for the schedule. To extend the component, modify the code
in this unit.  But remember, any code modified here will affect all schedules.

Users descend their own grid from TjwCustomSched and impliment the following
methods.
TjwDayView.BuildDisplayLine
TjwDayView.ExecuteQuery
TjwDayView.GetEventId

See TjwDayView in unit CustomSched for code that is used in the sample.
}

uses   Windows,  SysUtils, Graphics,  Classes, uConstants;

Type TjwEvent = class
  private
    fId         : Integer;
    fResourceId : String;
    fStartDate  : TDateTime;
    fEndDate    : TDateTime;
    fRect       : TRect;
    fFocused    : Boolean;
    fEventState : Integer;
    fText       : String;
    fFontColor  : TColor;
    fBackColor  : TColor;
    fDragging   : Boolean;
    fMode        :Integer;
    function GetLine(var Source:String):String;
  public
    constructor Create;
    procedure SetValues(aId:Integer; aText:String;
                aFontColor,aBackColor:TColor; aRect: TRect);
    procedure Draw(aCanvas: TCanvas);
    property Id: Integer read fId write fId;
    property ResourceId : String read fResourceId write fResourceId;
    property StartDate : TDateTime read fStartDate write fStartDate;
    property EndDate   : TDateTime read fEndDate write fEndDate;
    property Rect: TRect read fRect write fRect;
    property Focused: Boolean read fFocused write fFocused;
    property Dragging: Boolean read fDragging write fDragging;
    property EventState: Integer read fEventState write fEventState;
    property Text: String read fText write fText;
    property FontColor: TColor read fFontColor write fFontColor;
    property BackColor: TColor read fBackColor write fBackColor;
  end;

Type TjwEventList = class(TList)
  private
    fCanvas : TCanvas;
    fTempRect :TRect;
    procedure DrawOverLaps(aPosition:Integer);
  public
    Destructor Destroy; override;
    procedure ClearAll;
    procedure EventDelete(Id:Integer);
    procedure EventEdit(Id: Integer; aText:String;
                        aFontColor,aBackColor:TColor; aRect: TRect);
    procedure EventSetText(Id: Integer; aText:String);
    procedure EventSetState(Id: Integer; aValue: integer);
    procedure EventAdd(Id: Integer; aText:String; aState:Integer;
                       aFontColor,aBackColor:TColor; aRect: TRect);
    procedure EventSetRect(Id: Integer; aRect:TRect);
    function FindEvent(aRect :TRect):Integer;
    function SetDragging(aRect :TRect):Integer;
    procedure CancelDragging(Id: Integer);
    function EventGetRect(Id: Integer; var aRect :TRect):Boolean;
    procedure DrawEvents;
    procedure DrawEvent(CellRect: TRect;SetFocused: Boolean);
    procedure ReDrawEvent(Id:Integer);
    property Canvas: TCanvas read fCanvas write fCanvas;
  end;

implementation

//=================== TjwEvent ============================

constructor TjwEvent.Create;
begin
  fFocused := false;
  fDragging := false;
  fEventState := 0;
end;

function TjwEvent.GetLine(var Source:String):String;
var i:Integer;
begin
  i := Pos(SEP, Source);
  If i=0 then begin
    Result := Source;
    Source := '';
  end
  else begin
    Result := Copy(Source,1, i-1);
    Source := Copy(Source, i+1, Length(Source));
  end;
end;

procedure TjwEvent.SetValues(aId:Integer; aText:String; 
          aFontColor,aBackColor:TColor; aRect: TRect);
begin
  Id := aId;
  Text := aText;
  FontColor := aFontColor;
  BackColor := aBackColor;
  Rect := aRect;
end;

procedure TjwEvent.Draw(aCanvas:TCanvas);
var aRect:TRect;
    sText,sDisplayText:String;
    iHeight:Integer;
begin
  With aCanvas, aRect do begin
    Brush.Color := gEventColors[BackColor];
    Font.Color  := FontColor;
    Case EventState of
      0:Pen.Color := clRed;
      1:Pen.Color := clLime;
      2:Pen.Color := clfuchsia;
      else Pen.Color := clBlack;
    end;
    aRect := Rect;
    if Focused then begin
      Rectangle(Left, Top, Right, Bottom);
      InflateRect(aRect,-1,-1);
      Rectangle(Left, Top, Right, Bottom);
    end
    else Rectangle(Left, Top, Right, Bottom);
    Pen.Color := clBlack;
    iHeight := aCanvas.TextHeight('A');
    InflateRect(aRect, -3,-1);
    Bottom := Top + iHeight;
    sText := Text;
    While (Bottom < Rect.Bottom)and(sText <> '') do begin
      sDisplayText := GetLine(sText);
      TextRect(aRect, Left, Top, sDisplayText);
      Top := Top + iHeight;
      Bottom := Bottom + iHeight;
    end;
  end;
end;

//=================== TjwEventList ============================

destructor TjwEventList.Destroy;
begin
  ClearAll;
end;

function RectIsNull(const R: TRect): Boolean;
begin
  with R do
    Result := (Left = 0) and (Right = 0) and (Top = 0) and (Bottom = 0);
end;

procedure TjwEventList.DrawOverLaps(aPosition:Integer);
var i: Integer;
    aRect, bRect, cRect:TRect;
    aText:String;
begin
  bRect := TjwEvent(Items[aPosition]).Rect;
  For i := 0 to Count - 1 do
  With Canvas do
  Try
    if i <> aPosition then begin
      aRect := TjwEvent(Items[i]).Rect;
      IntersectRect(cRect,aRect,bRect);

      If (cRect.Left <> 0) then
      Try
        InflateRect(cRect,-3,-3);
        brush.Style := bsFDiagonal;
        brush.color := clBlack;
        fillrect(cRect);
        If (aRect.Top > bRect.Top) then
          aText := TjwEvent(Items[i]).Text
        else
          aText := TjwEvent(Items[aPosition]).Text;
        While (TextWidth(aText) > (cRect.Right - cRect.Left))
        and (Length(aText)>0) do
          aText := Copy(aText, 1, Length(aText)-1);  
        Textout( cRect.Left, cRect.Top, aText);
      Finally
        brush.style := bsSolid;
      End;
    end;
  Finally
  End;
end;

procedure TjwEventList.ClearAll;
begin
  While Count > 0 do begin
    TjwEvent(Items[0]).Free;
    Delete(0);
  end;
  Clear;
end;

procedure TjwEventList.EventDelete(Id:Integer);
var i:Integer;
begin
 For i := 0 to Count - 1 do
   If TjwEvent(Items[i]).Id = Id then begin
     TjwEvent(Items[i]).Free;
     Delete(i);
     Exit;
   end;
end;

procedure TjwEventList.EventEdit(Id: Integer; aText:String;
                        aFontColor,aBackColor:TColor; aRect: TRect);
var i:Integer;
begin
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
     TjwEvent(Items[i]).SetValues(Id, aText,aFontColor,aBackColor,aRect);
     Exit;
    end;
end;

procedure TjwEventList.EventSetText(Id: Integer; aText:String);
var i:Integer;
    aEvent:TjwEvent;
begin
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
     aEvent := TjwEvent(Items[i]);
     With aEvent do begin
       SetValues(Id, aText,FontColor,BackColor,Rect);
       Draw(Canvas);
     end;
     Exit;
    end;

end;

procedure TjwEventList.EventAdd(Id: Integer; aText:String; aState:Integer;
                        aFontColor,aBackColor:TColor; aRect: TRect);
begin
   Add(TjwEvent.Create);
   TjwEvent(Items[Count-1]).SetValues(Id, aText, aFontColor,aBackColor,aRect);
   TjwEvent(Items[Count-1]).EventState := aState;
end;

procedure TjwEventList.EventSetRect(Id: Integer; aRect:TRect);
var i:Integer;
begin
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
      TjwEvent(Items[i]).Rect := aRect;
      Break;
    end;
end;

procedure TjwEventList.EventSetState(Id: Integer; aValue: Integer);
var i:Integer;
begin
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
      TjwEvent(Items[i]).EventState := aValue;
      Break;
    end;
end;

function TjwEventList.FindEvent(aRect :TRect):Integer;
var i:Integer;
begin
  Result := -1;
  For i := 0 to Count - 1 do
    If bool(IntersectRect(fTempRect,aRect,TjwEvent(Items[i]).Rect)) then begin
      Result := TjwEvent(Items[i]).Id;
      Break;
    end;
end;

function TjwEventList.SetDragging(aRect :TRect):Integer;
var i:Integer;
begin
  Result := -1;
  For i := 0 to Count - 1 do
    If bool(IntersectRect(fTempRect,aRect,TjwEvent(Items[i]).Rect)) then begin
      Result := TjwEvent(Items[i]).Id;
      TjwEvent(Items[i]).Dragging := True;
      Break;
    end;
end;

procedure TjwEventList.CancelDragging(Id: Integer);
var i: Integer;
begin
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
      TjwEvent(Items[i]).Dragging := False;
      Break;
    end;
end;

function TjwEventList.EventGetRect(Id:Integer; var aRect :TRect):Boolean;
var i:Integer;
begin
  Result := False;
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
      aRect := TjwEvent(Items[i]).Rect;
      Result := True;
      Break;
    end;
end;

procedure TjwEventList.DrawEvents;
var i: Integer;
begin
  For i := 0 to Count - 1 do
  Try
    TjwEvent(Items[i]).Draw(Canvas);
    DrawOverLaps(i);
  Except
  End;
end;

procedure TjwEventList.DrawEvent(CellRect: TRect; SetFocused: Boolean);
var id, i :Integer;
begin
  id := FindEvent(CellRect);
  If id <> -1 then
  For i := 0 to Count - 1 do
  Try
    If SetFocused and (TjwEvent(Items[i]).Id <> id)
      and TjwEvent(Items[i]).Focused then begin
        TjwEvent(Items[i]).Focused := False;
        TjwEvent(Items[i]).Draw(Canvas);
        DrawOverLaps(i);
      end
    else If TjwEvent(Items[i]).Id = id then begin
      if SetFocused then TjwEvent(Items[i]).Focused := True;
      TjwEvent(Items[i]).Draw(Canvas);
      DrawOverLaps(i);
    end;
  Except
  End;
end;

procedure TjwEventList.ReDrawEvent(Id:Integer);
var i:Integer;
begin
  For i := 0 to Count - 1 do
    If TjwEvent(Items[i]).Id = Id then begin
      TjwEvent(Items[i]).Draw(Canvas);
      DrawOverLaps(i);
      Break;
    end;
end;



end.
